-----------------------------------------------------------------------------------------------
-- Client Lua Script for EGLink
-- Copyright (c) NCsoft. All rights reserved
--<ActionConfirmButtonReference>:SetActionData(GameLib.CodeEnumConfirmButtonType.CopyToClipboard, Text)
--<Control Class="ActionConfirmButton" <other misc button stuff>/>
-----------------------------------------------------------------------------------------------
 
require "Window"
require "ChatSystemLib"

-----------------------------------------------------------------------------------------------
-- EGLink Module Definition
-----------------------------------------------------------------------------------------------
local EGLink = {} 
 
-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
-- e.g. local kiExampleVariableMax = 999
local kstrTitle = "EGLink"
local kstrVersion = "0.4"
 
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function EGLink:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    -- initialize variables here
	o.tSaved = {}

    return o
end

function EGLink:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
		-- "UnitOrPackageName",
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end
 
--save addon status
function EGLink:OnSave(eLevel)
	if (eLevel ~= GameLib.CodeEnumAddonSaveLevel.Account) then 
		return 
	end 
	local l,t,r,b = self.wndMain:GetAnchorOffsets()

	return {
		tPosition_Main = { l = l, t = t, r = r, b = b },
		advice = self.tValidChannels[ChatSystemLib.ChatChannel_Advice],
		say = self.tValidChannels[ChatSystemLib.ChatChannel_Say],
		zone = self.tValidChannels[ChatSystemLib.ChatChannel_Zone],
		trade = self.tValidChannels[ChatSystemLib.ChatChannel_Advice],
	}
end

--restore addon status
function EGLink:OnRestore(eLevel,tSavedData) 
	if (eLevel ~= GameLib.CodeEnumAddonSaveLevel.Account) then 
		return 
	end 
	
	if tSavedData.advice ~= nil then
		self.tSaved.advice = tSavedData.advice
	end
	if tSavedData.say ~= nil then
		self.tSaved.say = tSavedData.say
	end
	if tSavedData.zone ~= nil then
		self.tSaved.zone = tSavedData.zone
	end
	if tSavedData.trade ~= nil then
		self.tSaved.trade = tSavedData.trade 
	end
	if tSavedData.tPosition_Main ~= nil then 
		self.tSaved.tPosition_Main = tSavedData.tPosition_Main
	end
end 

-----------------------------------------------------------------------------------------------
-- EGLink OnLoad
-----------------------------------------------------------------------------------------------
function EGLink:OnLoad()
    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("EGLink.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end

-----------------------------------------------------------------------------------------------
-- EGLink OnDocLoaded
-----------------------------------------------------------------------------------------------
function EGLink:OnDocLoaded()

	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
		self.wndMain = Apollo.LoadForm(self.xmlDoc, "EGLinkMain", nil, self)
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end
		self.wndChild = Apollo.LoadForm(self.xmlDoc, "EGLinkPopup", nil, self)
		if self.wndChild == nil then
			Apollo.AddAddonErrorText(self, "Could not load the child window for some reason.")
			return
		end

		self.tValidChannels = {
			[ChatSystemLib.ChatChannel_Whisper] 	= true,
			[ChatSystemLib.ChatChannel_Society] 	= true,
			[ChatSystemLib.ChatChannel_Party] 		= true,
			[ChatSystemLib.ChatChannel_Guild] 		= true,
			[ChatSystemLib.ChatChannel_Realm] 		= true,
			[ChatSystemLib.ChatChannel_Say] 		= false,
			[ChatSystemLib.ChatChannel_Zone] 		= false,
			[ChatSystemLib.ChatChannel_Trade] 		= false,
			[ChatSystemLib.ChatChannel_Advice] 		= false,
		}
		if self.tSaved.say then
			self.tValidChannels[ChatSystemLib.ChatChannel_Say] = true
			self.wndChild:FindChild("chkSay"):SetCheck(true)
		end
		if self.tSaved.zone then
			self.tValidChannels[ChatSystemLib.ChatChannel_Zone] = true
			self.wndChild:FindChild("chkZone"):SetCheck(true)
		end
		if self.tSaved.advice then
			self.tValidChannels[ChatSystemLib.ChatChannel_Advice] = true
			self.wndChild:FindChild("chkAdvice"):SetCheck(true)
		end
		if self.tSaved.trade then
			self.tValidChannels[ChatSystemLib.ChatChannel_Trade] = true
			self.wndChild:FindChild("chkTrade"):SetCheck(true)
		end
		if self.tSaved.tPosition_Main ~= nil then
			local tPosMain = self.tSaved.tPosition_Main
			self.wndMain:SetAnchorOffsets(tPosMain.l, tPosMain.t, tPosMain.r, tPosMain.b)
		end
		self.wndItemList = self.wndChild:FindChild("ItemList")
		self.wndChild:FindChild("Title"):SetText(kstrTitle .. " v" .. kstrVersion)
		--self.wndRecv = self.wndChild:FindChild("etRecv")
		self.wndURL = self.wndMain:FindChild("btnURL")
		self.wndURL:SetBGColor("white")
		-- item list
		self.wndMain:Show(true, true)
		self.wndChild:Show(false, true)
		Apollo.RegisterEventHandler("ChatMessage", "OnChatMessage", self)

		-- if the xmlDoc is no longer needed, you should set it to nil
		-- self.xmlDoc = nil
		
		-- Register handlers for events, slash commands and timer, etc.
		Apollo.RegisterSlashCommand("eglink", "OnEGLinkOn", self)


		-- Do additional Addon initialization here
	end
end

-----------------------------------------------------------------------------------------------
-- EGLink Functions
-----------------------------------------------------------------------------------------------
-- Define general functions here

-- on SlashCommand "/eglink"
function EGLink:OnEGLinkOn()
	self.wndChild:Invoke() -- show the window
end

function EGLink:OnChatMessage(channelCurrent, tMessage)
	--we don't want xml
	if tMessage.xml then
		return
	end
	-- we're not listening except on a few channels
	local eChannelType = channelCurrent:GetType()
	-- do we have a link?
	if self.tValidChannels[eChannelType] then
		local strSender, strLink = self:ParseForLinks(tMessage)
		if strLink and strSender then
			--ChatSystemLib.PostOnChannel( ChatSystemLib.ChatChannel_Command, strMessage, "" )
			self:AddMessage(strSender, strLink)
		end
	end
	
end

function EGLink:ParseForLinks(tMessage)
	for idx, tSegment in ipairs(tMessage.arMessageSegments) do
		if tSegment.strText then
			local i = string.find(tSegment.strText, "http://")
			if i == nil then
				i = string.find(tSegment.strText, "https://")
			end
			if i ~= nil then
				local strLink = string.sub(tSegment.strText, i)
				if strLink then
					local j = string.find(strLink, " ")
					if j ~= nil then
						strLink = string.sub(strLink, 1, (j-1))
					end
					return tMessage.strSender, strLink
				end
			end
		end
	end
	return nil, nil
end

function EGLink:AddMessage(strSender, strLink)
	--local text = self.wndRecv:GetText()
	--text = text .. "\n" .. strMessage
	--self.wndRecv:SetText(text)
	--self.wndRecv:SetVScrollPos(self.wndRecv:GetVScrollRange())
	if not self.wndChild:IsVisible() then
		self.wndURL:SetBGColor("red")
	end
	local wnd = Apollo.LoadForm(self.xmlDoc, "ListItem", self.wndItemList, self)
	local btn = wnd:FindChild("btnLink")
	btn:SetActionData(GameLib.CodeEnumConfirmButtonType.CopyToClipboard, strLink)
	btn:SetText(strSender .. ": " .. strLink)
	self.wndItemList:ArrangeChildrenVert()
	self.wndItemList:SetVScrollPos(self.wndItemList:GetVScrollRange())
end
-----------------------------------------------------------------------------------------------
-- EGLinkForm Functions
-----------------------------------------------------------------------------------------------
-- when the Cancel button is clicked
function EGLink:OnCancel()
	self.wndChild:Close() -- hide the window
end

function EGLink:OnClear()
	--self.wndRecv:SetText("")
	self.wndItemList:DestroyChildren()
	self.wndItemList:ArrangeChildrenVert()
end

function EGLink:OnCheck_Say( wndHandler, wndControl, eMouseButton )
	if wndHandler:IsChecked() then
		self.tValidChannels[ChatSystemLib.ChatChannel_Say] = true
	else
		self.tValidChannels[ChatSystemLib.ChatChannel_Say] = false
	end
end

function EGLink:OnCheck_Zone( wndHandler, wndControl, eMouseButton )
	if wndHandler:IsChecked() then
		self.tValidChannels[ChatSystemLib.ChatChannel_Zone] = true
	else
		self.tValidChannels[ChatSystemLib.ChatChannel_Zone] = false
	end
end

function EGLink:OnCheck_Trade( wndHandler, wndControl, eMouseButton )
	if wndHandler:IsChecked() then
		self.tValidChannels[ChatSystemLib.ChatChannel_Trade] = true
	else
		self.tValidChannels[ChatSystemLib.ChatChannel_Trade] = false
	end
end

function EGLink:OnCheck_Advice( wndHandler, wndControl, eMouseButton )
	if wndHandler:IsChecked() then
		self.tValidChannels[ChatSystemLib.ChatChannel_Advice] = true
	else
		self.tValidChannels[ChatSystemLib.ChatChannel_Advice] = false
	end
end

---------------------------------------------------------------------------------------------------
-- EGLinkMain Functions
---------------------------------------------------------------------------------------------------

function EGLink:OnURL( wndHandler, wndControl, eMouseButton )
	if self.wndChild:IsVisible() then
		self.wndChild:Close()
	else
		self.wndURL:SetBGColor("white")
		self.wndChild:Invoke()
	end
end

-----------------------------------------------------------------------------------------------
-- EGLink Instance
-----------------------------------------------------------------------------------------------
local EGLinkInst = EGLink:new()
EGLinkInst:Init()
